"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parse = void 0;
const runtime_1 = require("./runtime");
const transformers_1 = require("./transformers");
const utils_1 = require("./utils");
const parse = (program) => {
    runtime_1.context.stack.push(program);
    const keys = Object.keys(program.args);
    const args = keys.reduce((result, key) => {
        const argument = program.args[key];
        const value = (0, utils_1.getArgumentValue)(key.toString(), argument, program.argv);
        if ((0, utils_1.isNil)(argument.default) && (0, utils_1.isNil)(value) && !argument.optional) {
            const error = new Error(`Required argument option: ${key.toString()}`, {
                cause: {
                    key,
                    value
                }
            });
            runtime_1.context.errors.push(error);
        }
        const type = argument.type;
        if (type === String) {
            return { ...result, [key]: value };
        }
        if (type === Number) {
            return { ...result, [key]: (0, transformers_1.toNumber)(value) };
        }
        if (type === Date) {
            return { ...result, [key]: (0, transformers_1.toDate)(value) };
        }
        if (type === Boolean) {
            return { ...result, [key]: (0, transformers_1.toBoolean)(value, argument.default) };
        }
        return { ...result, [key]: (0, transformers_1.toObject)(type, value, argument.default) };
    }, {});
    const help = () => {
        const errors = runtime_1.context.errors.map(error => `- ${error.message}`).join('\r\n');
        console.error('collected errors:', errors);
        console.log('todo: should provide help for last called options:', runtime_1.context.stack.at(runtime_1.context.stack.length - 1));
        process.exit(1);
    };
    if (runtime_1.context.errors.length) {
        help();
    }
    return {
        args,
        help
    };
};
exports.parse = parse;
//# sourceMappingURL=parse.js.map